#!/usr/bin/env python
# coding=utf8
#
# This file is part of PlayStream - enigma2 plugin to play video streams from various sources
# Copyright (c) 2016 ivars777 (ivars777@gmail.com)
# Distributed under the GNU GPL v3. For full terms see http://www.gnu.org/licenses/gpl-3.0.en.html
#

import sys,os,re

class ContentSources(object):
    """Wrapper for content sources plugin"""

    #----------------------------------------------------------------------
    def __init__(self,plugin_path):
        self.plugins = {}
        self.error_content = [("..atpakaļ","back",None,"Kļūda, atgriezties atpakaļ")]
        sys.path.insert(0, plugin_path)
        for f in os.listdir(plugin_path):
            fname, ext = os.path.splitext(f)
            if ext == '.py':
                mod = __import__(fname)
                if "Source" in dir(mod):
                    self.plugins[fname] = mod.Source()
        sys.path.pop(0)
        
    def get_content(self,data):
        source = data.split("::")[0]
        if source in self.plugins:
            content = self.plugins[source].get_content(data)
            if content:
                if isinstance(content,list):      
                    for i,item in enumerate(content):
                        item2=[]
                        for el in item:
                            if isinstance(el,unicode):
                                el = el.encode("utf8")
                            item2.append(el)
                        content[i]=tuple(item2)
                else:
                    item2=[]
                    for el in content:
                        if isinstance(el,unicode):
                            el = el.encode("utf8")
                        item2.append(el)
                    content=tuple(item2)
                return content
            else:
                return self.error_content
        else:
            return self.error_content
        
    def is_video(self,data):
        source = data.split("::")[0]
        if source in self.plugins:
            return self.plugins[source].is_video(data)
        else:
            return False
    def options_read(self,source):
        if source in self.plugins:
            options = self.plugins[source].options_read()
            if options:
                return options
            else:
                return None
        else:
            return None
        
    def options_write(self,source,options):
        if source in self.plugins:
            return self.plugins[source].options_write(options)
        else:
            return None
    
if __name__ == "__main__":
    
    sources = ContentSources("sources")
    if len(sys.argv)>1:
        data= sys.argv[1]
    else:
        data = "config::home"
    
    history = []
    cur = ("Home",data,None,None)
    content = sources.get_content(cur[1])
    
    exit_loop = False
    while True:
        print
        for i,item in enumerate(content):
            s = "%i: %s - %s %s %s "%(i+1,item[0],item[1],item[2],item[3])
            print s #.encode(sys.stdout.encoding,"replace")
            
        while True:
            a = raw_input("Enter numeber, q for exit: ")
            if a in ("q","Q","x","X"): 
                exit_loop = True
                print "Exiting"
                break
            try:
                n = int(a)
                break
            except:
                print "Not number!"
        if exit_loop: break 
        cur2 = content[n-1]
        if cur2[1] == "back":
            cur = history.pop()
            
        elif cur2[1].startswith("http") or sources.is_video(cur2[1]):
            if sources.is_video(cur2[1]):
                cur2 = sources.get_content(cur2[1])
            if cur2[1].startswith("http") or cur2[1].startswith("rtmp"):
                from subprocess import call
                url = cur2[1]
                title = cur2[0]
                print "\n**Play %s - %s"%(title,url)
                call([r"c:\Program Files\VideoLAN\VLC\vlc.exe",url])
            else:
                print "**No stream to play - %s "%(cur2[0])
                raw_input("Press any key")
            #import os
            #os.system('"c:\Program Files (x86)\VideoLAN\VLC\vlc.exe" "%s"'%cur2[1])
        else:
            history.append(cur)
            cur = cur2
        content = sources.get_content(cur[1])
            
        
        