#!/usr/bin/env python
# coding=utf8
#
#
# This file is part of PlayStream - enigma2 plugin to play video streams from various sources
# Copyright (c) 2016 ivars777 (ivars777@gmail.com)
# Distributed under the GNU GPL v3. For full terms see http://www.gnu.org/licenses/gpl-3.0.en.html
# Used fragments of code from enigma2-plugin-tv3play by Taapat (https://github.com/Taapat/enigma2-plugin-tv3play)
#

import os,time,sys,os,os.path
import datetime

from enigma import ePicLoad, eServiceReference, eTimer
from Components.ActionMap import ActionMap
from Components.AVSwitch import AVSwitch
from Components.Label import Label
from Components.MenuList import MenuList
from Components.Pixmap import Pixmap
from Components.Sources.List import List
from Components.Sources.StaticText import StaticText
from Plugins.Plugin import PluginDescriptor
from Screens.InfoBar import MoviePlayer
from Screens.MessageBox import MessageBox
from Screens.Screen import Screen
from Tools.BoundFunction import boundFunction
from Tools.Directories import resolveFilename, SCOPE_PLUGINS
from Tools.LoadPixmap import LoadPixmap
from ContentSources import ContentSources
from twisted.web.client import downloadPage,defer,reactor


TMPDIR = "/tmp/playstream/"

class TVPlayer(MoviePlayer):
    def __init__(self, session, service):
        MoviePlayer.__init__(self, session, service)
        self.skinName = "MoviePlayer"

    def leavePlayer(self):
        self.close()
        #self.session.openWithCallback(self.leavePlayerConfirmed, MessageBox, _("Stop playing?"))

    def leavePlayerConfirmed(self, answer):
        if answer:
            self.close()

    def doEofInternal(self, playing):
        self.close()

    #def getPluginList(self):
        #from Components.PluginComponent import plugins
        #list = []
        #for p in plugins.getPlugins(where = PluginDescriptor.WHERE_EXTENSIONSMENU):
            #if p.name != _("TV Play"):
                #list.append(((boundFunction(self.getPluginName, p.name),
                    #boundFunction(self.runPlugin, p), lambda: True), None))
        #return list

    def showMovies(self):
        pass

class MainScreen(Screen):
    skin = """
<screen position="center,center" size="1015,590" type="skinned">
    <eLabel position="5,0" size="1000,2" backgroundColor="#aaaaaa" />
    <widget name="title" position="10,2" size="1000,38" font="Regular;30" />
    <widget source="list" render="Listbox" position="10,55" size="580,470" \
            scrollbarMode="showOnDemand" >
            <convert type="TemplatedMultiContent" >
            {
                    "template": [MultiContentEntryText(pos=(10, 1), size=(560, 30), \
                                    font=0, flags=RT_HALIGN_LEFT, text=0)],
                    "fonts": [gFont("Regular", 20)],
                    "itemHeight": 30
            }
            </convert>
    </widget>
    <widget name="pic" position="646,55" size="327,250" alphatest="on" />
    <widget name="cur" position="610,310" size="400,250" halign="center" font="Regular;20" />
    <eLabel position="110,540" size="148,2" backgroundColor="#00ff2525" />
    <eLabel position="370,540" size="148,2" backgroundColor="#00389416" />
    <widget source="key_red" backgroundColor="red" render="Label" position="110,540" zPosition="2" size="148,30" \
            valign="center" halign="center" font="Regular;22" transparent="1" />
    <widget source="key_green" backgroundColor="green" render="Label" position="370,540" zPosition="2" size="148,30" \
            valign="center" halign="center" font="Regular;22" transparent="1" />
</screen>"""

    def __init__(self, session):
        Screen.__init__(self, session)
        #self.setTitle("Play Stream")
        self.session = session
        self["key_red"] = StaticText(_("Back"))
        self["key_green"] = StaticText(_("Select"))
        self["actions"] = ActionMap(["OkCancelActions", "ColorActions","MenuActions", "NumberActions"],
                                    {
                                        "cancel": self.Cancel,
                                        "ok": self.Ok,
                                        "green": self.Ok,
                                        "red": self.Back,
                                        "menu": self.item_menu,
                                    })
        self["list"] = List([])
        self["list"].onSelectionChanged.append(self.SelectionChanged)
        self["pic"] = Pixmap()
        self["cur"] = Label()
        self["title"] = Label()
        self.downloading = 0
        if not os.path.exists(TMPDIR):
            os.mkdir(TMPDIR)
        self.onLayoutFinish.append(self.LayoutFinish)

    def LayoutFinish(self):
        self.cur_directory = os.path.dirname(os.path.realpath(__file__))
        self.defimage = LoadPixmap(os.path.join(self.cur_directory,"PlayStream.png"))
        #self.defimage = None
        #sc = AVSwitch().getFramebufferScale()
        #self.defimage0 = ePicLoad()
        #self.defimage0.PictureData.get().append(boundFunction(self.FinishDecodeDef, os.path.join(self.cur_directory,"PlayStream.png")))
        #self.defimage0.setPara((self["pic"].instance.size().width(),self["pic"].instance.size().height(),sc[0], sc[1], False, 0, "#00000000"))
        #self.defimage0.startDecode("default")
        
        self.images = {}
        self.images_url = {}
        self.picloads = {}
        self.history = []
        
        self.sources = ContentSources(os.path.join(self.cur_directory,"sources"))
        self.cur_menu = ("PlayStream","config::home","","Sākums")
        self.content = self.sources.get_content(self.cur_menu[1])
        print self.content
        self["list"].setList(self.content)
        self["cur"].setText(self.content[0][3])
        self.setTitle(self.cur_menu[0])
        self.ShowPic(self.content[0][2])


    def SelectionChanged(self):
        current = self["list"].getCurrent()
        print "[PlayStream] SelectionChanged: current=",current
        if not current: return
        self["cur"].setText(current[3]) if current[3] else self["cur"].setText("")
        if current[2]:
            self.ShowPic(current[2])
        else:
            self.ShowDefPic()

    def setTitle(self,title):
        print self.keys()
        self["title"].setText(title)
        
    def ShowDefPic(self):
        if self.defimage:
            self["pic"].instance.setPixmap(self.defimage)

    def ShowPic(self,image_url):
        if self.images.has_key(image_url):
            if self.images[image_url] in (-1,-2):
                return
            elif self.images[image_url] == -3:
                self.ShowDefPic()
                return
            else:
                self["pic"].instance.setPixmap(self.images[image_url])
        else:
            if image_url.startswith("http"):
                fname = image_url.replace(":","-").replace("/","_")
                image_path = os.path.join(TMPDIR, fname)
                self.download_image(image_path, image_url)
            else: # local file
                image_path = os.path.join(self.cur_directory,image_url)
                self.start_decode(image_path,image_url)
            
    def start_decode(self,image_path,image_url):
        self.images[image_url] = -2
        self.images_url[image_path] = image_url
        sc = AVSwitch().getFramebufferScale()
        if  not self.picloads.has_key(image_path):
            self.picloads[image_path] = ePicLoad()
            self.picloads[image_path].PictureData.get().append(boundFunction(self.FinishDecode, image_path))
            self.picloads[image_path].setPara((self["pic"].instance.size().width(),
                                          self["pic"].instance.size().height(),
                                          sc[0], sc[1], False, 0, "#00000000"))
            print image_path,image_url
            self.picloads[image_path].startDecode(image_path)

    def FinishDecode(self, image_path,picInfo = None):
        image_url = self.images_url[image_path]
        del self.images_url[image_path] #III
        self.images[image_url] = self.picloads[image_path].getData()
        self["pic"].instance.setPixmap(self.images[image_url])
        del self.picloads[image_path]
        if len(self.images)>30:
            del self.images[self.images.keys()[0]]
            # self.images.pop()
            
    #def FinishDecodeDef(self, image_path,picInfo = None):
    #    self.defimage = self.defimage0.getData()
    #    del self.defimage0
    #    self["pic"].instance.setPixmap(self.defimage)
        
    def download_image(self,image_path,image_url):
        print "Image download started",self.downloading,image_path,image_url
        self.downloading += 1
        self.images[image_url] = -1
        downloadPage(image_url, image_path).addCallback(boundFunction(self.downloadFinished, image_path,image_url)).addErrback(boundFunction(self.downloadFailed, image_path,image_url))

    def downloadFinished(self, image_path, image_url, result):
        self.downloading -= 1
        print "[TV Play] Image downloaded finished ",self.downloading,image_path, image_url,result
        self.start_decode(image_path,image_url)       

    def downloadFailed(self, image_path, image_url,result):
        self.downloading -= 1
        print "[TV Play] Image downloaded failed ",self.downloading,image_path, image_url,result
        self.images[image_url] = -3


    def Ok(self):
        current = self["list"].getCurrent()
        index = self["list"].getIndex()
        
        if current[1].startswith("http") or current[1].startswith("rstp://") or self.sources.is_video(current[1]):
            if self.sources.is_video(current[1]):
                try:
                    current = self.sources.get_content(current[1])
                    if not (current[1].startswith("http") or current[1].startswith("rstp://")):
                        self.session. open(MessageBox, "Can not play stream- %s"%current[0] , MessageBox.TYPE_INFO)
                        return
                except Exception,e:
                    self.session. open(MessageBox, "Can not play stream - %s"%str(e) , MessageBox.TYPE_INFO)
                    return
            self.playVideo(current)
            
        elif current[1] == "back":
            cur_menu_old = self.cur_menu
            self.cur_menu = self.history.pop()
            new_content = self.sources.get_content(self.cur_menu[1])
            try:
                index = zip(*new_content)[1].index(cur_menu_old[1])
            except:
                index = 0
            self.setTitle(self.cur_menu[0])
            self.show_content(new_content,index)
            
        else:
            print "selected=",current        
            self.history.append(self.cur_menu)
            self.cur_menu = current
            try:
                new_content = self.sources.get_content(self.cur_menu[1])
            except Exception,e:
                self.cur_menu = self.history.pop()                
                self.session. open(MessageBox, "Error - %s"%str(e) , MessageBox.TYPE_INFO)
                return
            self.setTitle(self.cur_menu[0])
            self.show_content(new_content)
 
    def Back(self):
        self["list"].setIndex(0)
        self.Ok()

    def show_content(self,content,index=0):
        self["list"].setList(content)
        self["list"].setIndex(index)
        self.SelectionChanged()

    def Cancel(self):
        #if os.path.exists(TMPDIR):
            #for name in os.listdir(TMPDIR):
                #os.remove(os.path.join(TMPDIR, name))
            #os.rmdir(TMPDIR)
        self.close()
            
    def playVideo(self, current):
        hlspath = current[1]
        print "[TV Play] Play:", current
        ref = eServiceReference(4097, 0, hlspath)
        ref.setName(current[0])
        self.session.open(TVPlayer, ref)
    
    def item_menu(self):
        print "\n[PlayStream] options\n"
        current = self["list"].getCurrent()
        #self.session. open(MessageBox, "Item options - %s"%current[0] , MessageBox.TYPE_INFO)
        args=[current,self.cur_menu]
        self.session.open(ItemMenuScreen,current,self.cur_menu)

class ItemMenuScreen(Screen):
    skin = '\n\t<screen position="350,100" size="420,400" title="Item menu" >\n\t\t<widget name="list" position="0,0" size="420,400" scrollbarMode="showOnDemand" />\n\t</screen>'

    def __init__(self, session, *args):
        Screen.__init__(self, session)
        self.session = session
        print "\n[PlayStream] ItemMenuScreen %s | %s\n"%(args[0][1],args[1][1])
        self['actions'] = ActionMap(['OkCancelActions'], {'ok': self.run,
         'cancel': self.close}, -1)
        #self["list"] = List([])
        lst = [
            ("Aditional information","info"),
            ("Add to bouquet","bouquet"),
            ("Add to favorites","favorites"),
            ("Download video","favorites")
        ]
        self['list'] = MenuList(lst)
        #self["list"].setList(lst)

    def run(self):
        current = self['list'].getCurrent()
        return

